@@ -1,12 +1,16 @@
-import consul
 import unittest
 
+from unittest.mock import Mock, patch, PropertyMock
+
+import consul
+
 from consul import ConsulException, NotFound
-from mock import Mock, PropertyMock, patch
+
 from patroni.dcs import get_dcs
-from patroni.dcs.consul import AbstractDCS, Cluster, Consul, ConsulInternalError, \
-    ConsulError, ConsulClient, HTTPClient, InvalidSessionTTL, InvalidSession, RetryFailedError
+from patroni.dcs.consul import AbstractDCS, Cluster, Consul, ConsulClient, ConsulError, \
+    ConsulInternalError, HTTPClient, InvalidSession, InvalidSessionTTL, RetryFailedError
 from patroni.postgresql.mpp import get_mpp
+
 from . import SleepException
 
 
@@ -42,8 +46,8 @@ def kv_get(self, key, **kwargs):
                   'ModifyIndex': 6429, 'Value': b'{"leader": "leader", "sync_standby": null}'},
                  {'CreateIndex': 1085, 'Flags': 0, 'Key': key + 'failsafe', 'LockIndex': 0,
                   'ModifyIndex': 6429, 'Value': b'{'},
-                 {'CreateIndex': 1085, 'Flags': 0, 'Key': key + 'status', 'LockIndex': 0,
-                  'ModifyIndex': 6429, 'Value': b'{"optime":4496294792, "slots":{"ls":12345}}'}])
+                 {'CreateIndex': 1085, 'Flags': 0, 'Key': key + 'status', 'LockIndex': 0, 'ModifyIndex': 6429,
+                  'Value': b'{"optime":4496294792,"slots":{"ls":12345},"retain_slots":["postgresql0","postgresql1"]}'}])
     if key == 'service/good/':
         return good_cls
     if key == 'service/broken/':
@@ -180,24 +184,24 @@ class TestConsul(unittest.TestCase):
     @patch.object(consul.Consul.Session, 'renew')
     @patch.object(consul.Consul.KV, 'put', Mock(side_effect=ConsulException))
     def test_update_leader(self, mock_renew):
-        leader = self.c.get_cluster().leader
+        cluster = self.c.get_cluster()
         self.c._session = 'fd4f44fe-2cac-bba5-a60b-304b51ff39b8'
         with patch.object(consul.Consul.KV, 'delete', Mock(return_value=True)):
             with patch.object(consul.Consul.KV, 'put', Mock(return_value=True)):
-                self.assertTrue(self.c.update_leader(leader, 12345, failsafe={'foo': 'bar'}))
+                self.assertTrue(self.c.update_leader(cluster, 12345, failsafe={'foo': 'bar'}))
             with patch.object(consul.Consul.KV, 'put', Mock(side_effect=ConsulException)):
-                self.assertFalse(self.c.update_leader(leader, 12345))
+                self.assertFalse(self.c.update_leader(cluster, 12345))
             with patch('time.time', Mock(side_effect=[0, 0, 0, 0, 100, 200, 300])):
-                self.assertRaises(ConsulError, self.c.update_leader, leader, 12345)
+                self.assertRaises(ConsulError, self.c.update_leader, cluster, 12345)
         with patch('time.time', Mock(side_effect=[0, 100, 200, 300])):
-            self.assertRaises(ConsulError, self.c.update_leader, leader, 12345)
+            self.assertRaises(ConsulError, self.c.update_leader, cluster, 12345)
         with patch.object(consul.Consul.KV, 'delete', Mock(side_effect=ConsulException)):
-            self.assertFalse(self.c.update_leader(leader, 12347))
+            self.assertFalse(self.c.update_leader(cluster, 12347))
         mock_renew.side_effect = RetryFailedError('')
         self.c._last_session_refresh = 0
-        self.assertRaises(ConsulError, self.c.update_leader, leader, 12346)
+        self.assertRaises(ConsulError, self.c.update_leader, cluster, 12346)
         mock_renew.side_effect = ConsulException
-        self.assertFalse(self.c.update_leader(leader, 12347))
+        self.assertFalse(self.c.update_leader(cluster, 12347))
 
     @patch.object(consul.Consul.KV, 'delete', Mock(return_value=True))
     def test_delete_leader(self):
@@ -258,9 +262,8 @@ class TestConsul(unittest.TestCase):
         d['state'] = 'running'
         d['role'] = 'bla'
         self.assertIsNone(self.c.update_service({}, d))
-        for role in ('master', 'primary'):
-            d['role'] = role
-            self.assertTrue(self.c.update_service({}, d))
+        d['role'] = 'primary'
+        self.assertTrue(self.c.update_service({}, d))
 
     @patch.object(consul.Consul.KV, 'put', Mock(side_effect=ConsulException))
     def test_reload_config(self):
